/*
  ==============================================================================

    This file was auto-generated!

    It contains the basic framework code for a JUCE plugin processor.

  ==============================================================================
*/

#include "PluginProcessor.h"
#include "PluginEditor.h"

#define PARAM_SETNAME "StereoWidthParams"
#define PARAM_BYPASS "Bypass"
#define PARAM_WIDTH	"StereoWidth"
//==============================================================================
StereoWidthCtrlAudioProcessor::StereoWidthCtrlAudioProcessor()
{
	//Param Handling
	addParameter(BoolParam[BP_BYPASS] = new AudioParameterBool(PARAM_BYPASS, PARAM_BYPASS, 0));
	addParameter(FloatParam[FP_WIDTH] = new AudioParameterFloat(PARAM_WIDTH, PARAM_WIDTH, 0.0f, 5.0, mWidthControl.Getwidth()));

	//start listening for changes
	MajorParamChange = true;//start it off with deep updates regardless
	addListener(this);//making the processor it's own parameter listener 
					  //(add conditional checks to the callback, but no deep changes - save those for process block)
}

StereoWidthCtrlAudioProcessor::~StereoWidthCtrlAudioProcessor()
{
	removeListener(this);
}

//==============================================================================
const String StereoWidthCtrlAudioProcessor::getName() const
{
    return JucePlugin_Name;
}

bool StereoWidthCtrlAudioProcessor::acceptsMidi() const
{
   #if JucePlugin_WantsMidiInput
    return true;
   #else
    return false;
   #endif
}

bool StereoWidthCtrlAudioProcessor::producesMidi() const
{
   #if JucePlugin_ProducesMidiOutput
    return true;
   #else
    return false;
   #endif
}

double StereoWidthCtrlAudioProcessor::getTailLengthSeconds() const
{
    return 0.0;
}

int StereoWidthCtrlAudioProcessor::getNumPrograms()
{
    return 1;   // NB: some hosts don't cope very well if you tell them there are 0 programs,
                // so this should be at least 1, even if you're not really implementing programs.
}

int StereoWidthCtrlAudioProcessor::getCurrentProgram()
{
    return 0;
}

void StereoWidthCtrlAudioProcessor::setCurrentProgram (int index)
{
}

const String StereoWidthCtrlAudioProcessor::getProgramName (int index)
{
    return String();
}

void StereoWidthCtrlAudioProcessor::changeProgramName (int index, const String& newName)
{
}

//==============================================================================
void StereoWidthCtrlAudioProcessor::prepareToPlay (double sampleRate, int samplesPerBlock)
{
    // Use this method as the place to do any pre-playback
    // initialisation that you need..
}

void StereoWidthCtrlAudioProcessor::releaseResources()
{
    // When playback stops, you can use this as an opportunity to free up any
    // spare memory, etc.
}

#ifndef JucePlugin_PreferredChannelConfigurations
bool StereoWidthCtrlAudioProcessor::setPreferredBusArrangement (bool isInput, int bus, const AudioChannelSet& preferredSet)
{
    // Reject any bus arrangements that are not compatible with your plugin

    const int numChannels = preferredSet.size();

   #if JucePlugin_IsMidiEffect
    if (numChannels != 0)
        return false;
   #elif JucePlugin_IsSynth
    if (isInput || (numChannels != 1 && numChannels != 2))
        return false;
   #else
    if (numChannels != 1 && numChannels != 2)
        return false;

    if (! AudioProcessor::setPreferredBusArrangement (! isInput, bus, preferredSet))
        return false;
   #endif

    return AudioProcessor::setPreferredBusArrangement (isInput, bus, preferredSet);
}
#endif

void StereoWidthCtrlAudioProcessor::processBlock (AudioSampleBuffer& buffer, MidiBuffer& midiMessages)
{
	if (MajorParamChange)
	{
		//this is just an example (not really major), but this is where you would put code to reconfigure buffers or other major changes.
		mWidthControl.SetWidth((*FloatParam[FP_WIDTH]));
		MajorParamChange = false;
	}
	
    const int totalNumInputChannels  = getTotalNumInputChannels();
    const int totalNumOutputChannels = getTotalNumOutputChannels();

	if (totalNumInputChannels != 2 || totalNumOutputChannels != 2
		|| (*BoolParam[BP_BYPASS]))
	{
	}/*Nothing to do here - processing is in-place, so doing nothing is pass-through (for NumInputs=NumOutputs)*/
	else
	{//Not bypassed - do processing!
		float** pChannelData = buffer.getArrayOfWritePointers();
		mWidthControl.Process(pChannelData[0], pChannelData[1], buffer.getNumSamples());
	}
}

//==============================================================================
bool StereoWidthCtrlAudioProcessor::hasEditor() const
{
    return true; // (change this to false if you choose to not supply an editor)
}

AudioProcessorEditor* StereoWidthCtrlAudioProcessor::createEditor()
{
    return new StereoWidthCtrlAudioProcessorEditor (*this);
}

//==============================================================================
String FloatArrayToString(float* fData, int numFloat)
{//Return String of multiple float values separated by commas 
	String result = "";
	if (numFloat < 1)
		return result;
	for (int i = 0; i < (numFloat - 1); i++)
		result << String(fData[i]) << ",";//Use juce::String initializer for each value
	result << String(fData[numFloat - 1]);
	return result;
}
int StringToFloatArray(String sFloatCSV, float* fData, int maxNumFloat)
{//Return is number of floats copied to the fData array
 //-1 if there were more in the string than maxNumFloat
	StringArray Tokenizer;
	int TokenCount = Tokenizer.addTokens(sFloatCSV, ",", "");
	int resultCount = (maxNumFloat <= TokenCount) ? maxNumFloat : TokenCount;
	for (int i = 0; i < resultCount; i++)//only go as far as resultCount for valid data
		fData[i] = Tokenizer[i].getFloatValue();//fill data using String class float conversion
	return ((TokenCount <= maxNumFloat) ? resultCount : -1);
}
String IntArrayToString(int* IData, int numInt)
{//Return String of multiple float values separated by commas 
	String result = "";
	if (numInt < 1)
		return result;
	for (int i = 0; i < (numInt - 1); i++)
		result << String(IData[i]) << ",";//Use juce::String initializer for each value
	result << String(IData[numInt - 1]);
	return result;
}
int StringToIntArray(String sIntCSV, int* IData, int maxNumInt)
{//Return is number of floats copied to the fData array
 //-1 if there were more in the string than maxNumFloat
	StringArray Tokenizer;
	int TokenCount = Tokenizer.addTokens(sIntCSV, ",", "");
	int resultCount = (maxNumInt <= TokenCount) ? maxNumInt : TokenCount;
	for (int i = 0; i < resultCount; i++)//only go as far as resultCount for valid data
		IData[i] = Tokenizer[i].getIntValue();//fill data using String class float conversion
	return ((TokenCount <= maxNumInt) ? resultCount : -1);
}


void StereoWidthCtrlAudioProcessor::getStateInformation (MemoryBlock& destData)
{
	//Collect data you want to store
	for (int i = 0; i < NUM_FLOAT_PARAM; i++)
		lastStateData_Float[i] = (*FloatParam[i]);
	//write as xml
	XmlElement root(PARAM_SETNAME);
	XmlElement *el = root.createNewChildElement("FloatParam");
	el->addTextElement(String(FloatArrayToString(lastStateData_Float, NUM_FLOAT_PARAM)));

	el = root.createNewChildElement("BoolParam");
	for (int i = 0; i < NUM_BOOL_PARAM; i++)
		lastStateData_Bool[i] = (*BoolParam[i]);
	el->addTextElement(String(IntArrayToString(lastStateData_Bool, NUM_BOOL_PARAM)));

	copyXmlToBinary(root, destData);

}

void StereoWidthCtrlAudioProcessor::setStateInformation (const void* data, int sizeInBytes)
{
	XmlElement* pRoot = getXmlFromBinary(data, sizeInBytes);
	if (pRoot != NULL)
	{
		if (!pRoot->hasTagName(PARAM_SETNAME))
			return;

		forEachXmlChildElement((*pRoot), pChild)
		{
			if (pChild->hasTagName("FloatParam"))
			{
				String sFloatCSV = pChild->getAllSubText();
				if (StringToFloatArray(sFloatCSV, lastStateData_Float, NUM_FLOAT_PARAM) == NUM_FLOAT_PARAM)
				{//We have a new set, set with any conversions (via setParameter)
					for (int i = 0; i < NUM_FLOAT_PARAM; i++)
						*FloatParam[i] = lastStateData_Float[i];
				}
				else {};//ignore... or you could also call a "setUserParamDefaults" if desired.
			}
			if (pChild->hasTagName("BoolParam"))
			{
				String sIntCSV = pChild->getAllSubText();
				if (StringToIntArray(sIntCSV, lastStateData_Bool, NUM_BOOL_PARAM) == NUM_BOOL_PARAM)
				{//We have a new set, set with any conversions (via setParameter)
					for (int i = 0; i < NUM_BOOL_PARAM; i++)
						*BoolParam[i] = (lastStateData_Bool[i] !=0);
				}
				else {};//ignore... or you could also call a "setUserParamDefaults" if desired.
			}
		}
		delete pRoot;
	}
}

//==============================================================================
// This creates new instances of the plugin..
AudioProcessor* JUCE_CALLTYPE createPluginFilter()
{
    return new StereoWidthCtrlAudioProcessor();
}
